//@***h* BDBP/gb_dialogutils.pas
// 
// NAME
//   gb_dialogutils.pas
//
// PURPOSE
//  Provides routines for saving dialog form settings to file, reading them 
//  from a file etc. 
//
// DESCRIPTION
//  This unit provides routines for saving dialog form settings to file, 
//  reading them from a file etc.  
//
//  
// AUTHOR
//  Birgit Hassler, CIRES / NOAA ESRL CSD
//
// COPYRIGHT 
//   THIS SOFTWARE AND ITS DOCUMENTATION ARE CONSIDERED TO BE IN THE PUBLIC
//   DOMAIN AND THUS ARE AVAILABLE FOR UNRESTRICTED PUBLIC USE. THEY ARE
//   FURNISHED "AS IS." THE AUTHORS, THE UNITED STATES GOVERNMENT, ITS
//   INSTRUMENTALITIES, OFFICERS, EMPLOYEES, AND AGENTS MAKE NO WARRANTY,
//   EXPRESS OR IMPLIED, AS TO THE USEFULNESS OF THE SOFTWARE AND
//   DOCUMENTATION FOR ANY PURPOSE. THEY ASSUME NO RESPONSIBILITY (1) FOR
//   THE USE OF THE SOFTWARE AND DOCUMENTATION; OR (2) TO PROVIDE TECHNICAL
//   SUPPORT TO USERS.
//
// REVISION HISTORY
//   6/19/2014 Initial Version prepared for NCDC.
// 
//@***** 


unit gb_dialogutils;

{$I UNITDIREC.INC}

interface

uses
  Forms;

procedure ConfirmDirectory(var Sender:string;AllowCreation,AllowNil:boolean); overload;
procedure ConfirmDirectory(Sender:TObject;AllowCreation,AllowNil:boolean); overload;
procedure SaveDlgSettingsToFile(Filename:string;Sender:TForm);
procedure LoadDlgSettingsFromFile(Filename:string;var Sender:TForm);


implementation

uses
  comctrls,
  controls,
  checklst,
  dialogs,
  extctrls,
  grids,
  mask,
  stdctrls,
  sysutils,
  gb_files,
  gb_strings;

const
  {$IFDEF VER130} DelphiVersion=5; {$ENDIF}
  {$IFDEF VER140} DelphiVersion=6; {$ENDIF}
  {$IFDEF VER150} DelphiVersion=7; {$ENDIF}
  {$IFDEF VER160} DelphiVersion=8; {$ENDIF}
  {$IFDEF VER170} DelphiVersion=2005; {$ENDIF}
  {$IFDEF VER180} DelphiVersion=2005; {$ENDIF}

procedure ConfirmDirectory(var Sender:string;AllowCreation,AllowNil:boolean);
var
  Year:word;
  DirecOK:boolean;
begin
  DirecOK:=DirectoryExists(Sender);
  if ((not DirecOK) and (pos('YYYY',Sender)>0)) then
  begin
    Year:=1000;
    repeat inc(Year) until (DirectoryExists(InsertYear(Sender,inttostr(Year))) or (Year=3000));
    if DirectoryExists(InsertYear(Sender,inttostr(Year))) then DirecOK:=true;
  end;
  if not DirecOK then
  begin
    if (AllowNil and (Sender='')) then Exit else
    begin
      if AllowCreation then
      begin
        if MessageDlg('The directory '+Sender+' doesn''t exist. Do you want to create it?',
        mtConfirmation,[mbYes,mbNo],0)=mrYes then ForceDirectories(Sender);
      end
      else
        Sender:=InputBox('The directory '+Sender+' doesn''t exist, please re-enter the directory name',
          'Directory:',Sender);
    end;
  end;
end; {End of ConfirmDirectory.}

procedure ConfirmDirectory(Sender:TObject;AllowCreation,AllowNil:boolean);
var
  Direc:string;
begin
  if Sender is TLabeledEdit then Direc:=(Sender as TLabeledEdit).Text;
  if Sender is TEdit then Direc:=(Sender as TEdit).Text;
  ConfirmDirectory(Direc,AllowCreation,AllowNil);
  if Sender is TLabeledEdit then (Sender as TLabeledEdit).Text:=Direc;
  if Sender is TEdit then (Sender as TEdit).Text:=Direc;
end;

procedure WriteSettingsToFile(var Dest:textfile;Sender:TForm);
var
  I,J,K:integer; {Loop control variables.}
begin
  with Sender do
  for I:=0 to ComponentCount-1 do
  if pos('_nosave',Components[I].Name)=0 then
  begin
    if Components[I].ClassName='TEdit' then writeln(Dest,Components[I].Name,
      ' = ',(Components[I] as TEdit).Text);
    if Components[I].ClassName='TLabeledEdit' then writeln(Dest,Components[I].Name,
      ' = ',(Components[I] as TLabeledEdit).Text);
    if Components[I].ClassName='TMaskEdit' then writeln(Dest,Components[I].Name,
      ' = ',(Components[I] as TMaskEdit).EditText);
    if Components[I].ClassName='TCheckBox' then writeln(Dest,Components[I].Name,
      ' = ',BooleanToYesNo((Components[I] as TCheckBox).Checked,true));
    if Components[I].ClassName='TRadioButton' then writeln(Dest,Components[I].Name,
      ' = ',BooleanToYesNo((Components[I] as TRadioButton).Checked,true));
    if Components[I].ClassName='TCheckListBox' then with Components[I] as TCheckListBox do
    begin
      writeln(Dest,Name,' = ',Items.Count);
      for J:=0 to Items.Count-1 do writeln(Dest,'  '+Items[J]+' = ',BooleanToYesNo(Checked[J],true));
    end;
    if Components[I].ClassName='TListBox' then with Components[I] as TListBox do
    begin
      writeln(Dest,Name,' = ',Items.Count);
      for J:=0 to Items.Count-1 do writeln(Dest,'  '+Items[J]+' = ',BooleanToYesNo(Selected[J],true));
    end;
    if Components[I].ClassName='TComboBox' then with Components[I] as TComboBox do
    begin
      writeln(Dest,Name,' = ',Items.Count,' ',ItemIndex);
      for J:=0 to Items.Count-1 do writeln(Dest,'   '+Items[J]);
    end;
    if Components[I].ClassName='TRadioGroup' then with Components[I] as TRadioGroup do
      for J:=0 to ControlCount-1 do writeln(Dest,Items[J]+' = ',BooleanToYesNo((Controls[J]
        as TRadioButton).Checked,true));
    if Components[I].ClassName='TMemo' then with Components[I] as TMemo do
    begin
      writeln(Dest,Name,' = ',Lines.Count);
      for J:=0 to Lines.Count-1 do writeln(Dest,'  '+Name+'_Line[',J,']=',Lines[J]);
    end;
    if Components[I].ClassName='TStringGrid' then with Components[I] as TStringGrid do
    begin
      writeln(Dest,Name,' = ',ColCount,' ',RowCount);
      for J:=0 to ColCount-1 do
        for K:=0 to RowCount-1 do
          writeln(Dest,'  '+Name+'_Item[',J,',',K,']=',Cells[J,K]);
    end;
    if Components[I].ClassName='TDateTimePicker' then
    begin
      if (Components[I] as TDateTimePicker).Kind=dtkDate then
        writeln(Dest,(Components[I] as TDateTimePicker).Name,' = ',
        DateToStr((Components[I] as TDateTimePicker).DateTime));
      if (Components[I] as TDateTimePicker).Kind=dtkTime then
        writeln(Dest,(Components[I] as TDateTimePicker).Name,' = ',
        TimeToStr((Components[I] as TDateTimePicker).DateTime));
    end;
  end;
  writeln(Dest);
end;

procedure SaveDlgSettingsToFile(Filename:string;Sender:TForm);
var
  Source,        {File handle for package default file.}
  Dest:textfile; {File handle for temporary storage file.}
  HeaderString,  {The string denoting the header in the default file.}
  Linein:string; {Text read in from default file.}
begin
  HeaderString:=UpperCase('['+Sender.Name+']');
  if fileexists(Filename) then
  begin
    assignfile(Source,Filename);
    reset(Source);
    assignfile(Dest,FindJunkFile);
    rewrite(Dest);
    repeat
      readln(Source,Linein);
      writeln(Dest,Linein);
    until (eof(Source) or (uppercase(copy(Linein,1,length(HeaderString)))=HeaderString));
    if eof(Source) then writeln(Dest,HeaderString);
    WriteSettingsToFile(Dest,Sender);
    if not eof(Source) then
    begin
      repeat readln(Source,Linein) until (eof(Source) or (Linein=''));
      if not eof(Source) then
      repeat
        readln(Source,Linein);
        writeln(Dest,Linein);
      until eof(Source);
    end;
    closefile(Source);
    closefile(Dest);
    erase(Source);
    rename(Dest,Filename);
  end
  else
  begin
    assignfile(Dest,Filename);
    rewrite(Dest);
    writeln(Dest,HeaderString);
    WriteSettingsToFile(Dest,Sender);
    closefile(Dest);
  end;
end; {End of SaveDlgSettingsToFile.}

procedure LoadDlgSettingsFromFile(Filename:string;var Sender:TForm);
var
  DefaultFile,     {File handle for temporary storage file from which values are read.}
  Source:textfile; {The file handle for the default file.}
  StoreStr,        {Temporary storage string.}
  HeaderString,    {The string denoting the header in the default file.}
  Linein:string;   {Lines of text read in from the default file.}
  Hold:boolean;    {Temporary storage variable.}
  I,J,K:integer;   {General purpose loop control variables.}
  Test:string;
begin
  HeaderString:=UpperCase('['+Sender.Name+']');
  if fileexists(Filename) then
  begin
    assignfile(Source,Filename);
    reset(Source);
    repeat readln(Source,Linein) until
      ((uppercase(copy(Linein,1,length(HeaderString)))=HeaderString) or eof(Source));
    if not eof(Source) then
    begin
      assignfile(DefaultFile,'default.txt');
      rewrite(DefaultFile);
      repeat
        readln(Source,Linein);
        writeln(DefaultFile,Linein);
      until ((Linein='') or eof(Source));
      closefile(Source);
      with Sender do
      for I:=0 to ComponentCount-1 do
      if pos('_nosave',Components[I].Name)=0 then
      begin
        Test:=Components[I].ClassName;
        if Components[I].ClassName='TEdit' then (Components[I] as TEdit).Text:=
          GetPrompt(DefaultFile,Components[I].Name);
        if Components[I].ClassName='TLabeledEdit' then (Components[I] as TLabeledEdit).Text:=
          GetPrompt(DefaultFile,Components[I].Name);
        if Components[I].ClassName='TMaskEdit' then (Components[I] as TMaskEdit).Text:=
          GetPrompt(DefaultFile,Components[I].Name);
        if Components[I].ClassName='TDateTimePicker' then
        begin
          StoreStr:=GetPrompt(DefaultFile,Components[I].Name);
          if StoreStr<>'' then
          begin
            if (Components[I] as TDateTimePicker).Kind=dtkDate then
              (Components[I] as TDateTimePicker).DateTime:=StrToDate(StoreStr);
            if (Components[I] as TDateTimePicker).Kind=dtkTime then
              (Components[I] as TDateTimePicker).DateTime:=StrToTime(StoreStr);
          end;
        end;
        if Components[I].ClassName='TComboBox' then with Components[I] as TComboBox do
        begin
          StoreStr:=GetPrompt(DefaultFile,Name);
          if StoreStr<>'' then
          begin
            K:=strtoint(GetWord(StoreStr,' ',1));
            Items.Clear;
            for J:=0 to K-1 do
            begin
              readln(DefaultFile,Linein);
              Items.Add(Trim(Linein));
            end;
            ItemIndex:=strtoint(GetWord(StoreStr,' ',2));
            if ItemIndex<0 then ItemIndex:=0;
          end;
        end;
        if Components[I].ClassName='TCheckBox' then if GetYNPrompt(DefaultFile,
          Components[I].Name,Hold)=0 then (Components[I] as TCheckBox).Checked:=Hold;
        if Components[I].ClassName='TRadioButton' then if GetYNPrompt(DefaultFile,
          Components[I].Name,Hold)=0 then (Components[I] as TRadioButton).Checked:=Hold;
        if Components[I].ClassName='TListBox' then with Components[I] as TListBox do
        begin
          StoreStr:=GetPrompt(DefaultFile,Name);
          if StoreStr<>'' then
          begin
            K:=strtoint(GetWord(StoreStr,' ',1));
            Items.Clear;
            for J:=0 to K-1 do
            begin
              readln(DefaultFile,Linein);
              Items.Add(copy(Linein,3,pos('=',Linein)-4));
              if UpperCase(copy(Linein,pos('=',Linein)+2,10))='YES' then Selected[J]:=true
              else Selected[J]:=false;
            end;
          end;
        end;
        if Components[I].ClassName='TCheckListBox' then with Components[I] as TCheckListBox do
        begin
          StoreStr:=GetPrompt(DefaultFile,Name);
          if StoreStr<>'' then
          begin
            K:=strtoint(GetWord(StoreStr,' ',1));
            Items.Clear;
            for J:=0 to K-1 do
            begin
              readln(DefaultFile,Linein);
              Items.Add(copy(Linein,3,pos('=',Linein)-4));
              if UpperCase(copy(Linein,pos('=',Linein)+2,10))='YES' then Checked[J]:=true
              else Checked[J]:=false;
            end;
          end;
        end;
        if Components[I].ClassName='TRadioGroup' then with Components[I] as TRadioGroup do
          for J:=0 to ControlCount-1 do if GetYNPrompt(DefaultFile,Items[J],Hold)=0 then
            (Controls[J] as TRadioButton).Checked:=Hold;
        if Components[I].ClassName='TMemo' then with Components[I] as TMemo do
        begin
          StoreStr:=GetPrompt(DefaultFile,Name);
          if StoreStr<>'' then
          begin
            K:=strtoint(GetWord(StoreStr,' ',1));
            Lines.Clear;
            for J:=0 to K-1 do
            begin
              readln(DefaultFile,Linein);
              if copy(Linein,3,length(Name))=Name then
                Lines.Add(copy(Linein,pos('=',Linein)+1,length(Linein)));
            end;
          end;
        end;
        if Components[I].ClassName='TStringGrid' then with Components[I] as TStringGrid do
        begin
          StoreStr:=GetPrompt(DefaultFile,Name);
          if StoreStr<>'' then
          begin
            ColCount:=strtoint(GetWord(StoreStr,' ',1));
            RowCount:=strtoint(GetWord(StoreStr,' ',2));
            for J:=0 to ColCount-1 do
              for K:=0 to RowCount-1 do
              begin
                readln(DefaultFile,Linein);
                if copy(Linein,3,length(Name))=Name then
                  Cells[J,K]:=copy(Linein,pos('=',Linein)+1,length(Linein));
              end;
          end;
        end;
      end;
      closefile(DefaultFile);
      erase(DefaultFile);
    end
    else closefile(Source);
  end;
end;

end.
